function [mu,q,E] = FindElasticMean(Data)

close all;

% Maximum number of iterations
Niter=10;

% Resample all data according to arc-length with 100 points; compute the
% SRVFs of all curves in the data
n=size(Data,3); 
for i=1:n
    X = ReSampleCurve(Data(:,:,i),100);
    q(:,:,i) = curve_to_q(X);
end

% Set step size, gradient threshold and initialize algorithm
del = 1;
eps = 0.025;
mu = q(:,:,1);
iter=1;
E=1000;

% Display the initial estimate
pinit = q_to_curve(mu);
figure(100);clf
plot(pinit(1,:),pinit(2,:),'r','LineWidth',2)
axis equal off
title('Evolution of the Mean Shape','FontSize',20)

% Run iterative algorithm
while (iter<=Niter && E(iter)>eps)

    iter=iter+1;
    % Compute inverse exponential map at the current estimate of the mean
    vm = 0;    
    for i=1:n
        [iter i]
        v = ElasticShootingVector(mu,q(:,:,i),1);
        vm = vm + v;
    end
    
    % Compute the gradient
    vm = vm/n;
    
    % Compute the updated estimate of the mean
    E(iter) = sqrt(InnerProd_Q(vm,vm));
    mu = ElasticShooting(mu,del*vm);

    % Display the current estimate of the mean (blue) compared to the
    % initialization (red)
    p = q_to_curve(mu);
    figure(100);clf;hold on
    plot(pinit(1,:),pinit(2,:),'r','LineWidth',2)
    plot(p(1,:),p(2,:),'LineWidth',2)
    axis equal off
    title('Evolution of the Mean Shape','FontSize',20)
end

% Display the evolution of the gradient as a function of the iterations
E=E(2:end);
figure(101); 
plot(E,'LineWidth',2);
title('Energy Gradient','FontSize',20)
set(gca,'FontSize',18)

% Display the mean shape
figure(1); clf; 
p = q_to_curve(mu);
plot(p(1,:),p(2,:),'LineWidth',2);
axis equal off;
title('Mean Shape','FontSize',20)